window.Gleam.initSearch([{"doc":"storail","title":"storail","content":"# Stóráil\n\nA simple on-disc JSON based data store.\n\n[![Package Version](https://img.shields.io/hexpm/v/storail)](https://hex.pm/packages/storail)\n[![Hex Docs](https://img.shields.io/badge/hex-docs-ffaff3)](https://hexdocs.pm/storail/)\n\nA super basic on-disc data store that uses JSON files to persist data.\n\nIt doesn&#39;t have transactions, MVCC, or anything like that. It&#39;s just\nwriting files to disc. Useful for tiny little projects, and for fun.\n\n```sh\ngleam add storail@2\n```\n```gleam\nimport storail\nimport my_app\n\npub fn main() {\n  // Construct config to specify where your data is written to.\n  let config = storail.Config(\n    data_directory: &quot;/data&quot;,\n    temporary_directory: &quot;/tmp/storail&quot;,\n  )\n\n  // Define a collection for a data type in your application.\n  let cats = storail.Collection(\n    name: &quot;cats&quot;, \n    to_json: my_app.cat_to_json,\n    decoder: my_app.cat_decoder,\n    config:,\n  )\n\n  // A key points to a specific object within the collection, which \n  // may or may not yet exist.\n  let key = storail.key(collection, &quot;nubi&quot;)\n\n  // Write some data\n  let assert Ok(Nil) = storail.write(key, my_app.Cat(&quot;Nubi&quot;, 5))\n\n  // Read some data\n  storail.read(key)\n  // -&gt; Ok(my_app.Cat(&quot;Nubi&quot;, 5))\n}\n```\n\nFurther documentation can be found at &lt;https://hexdocs.pm/storail&gt;.\n","url":"index.html"},{"doc":"storail","title":"Collection","content":"pub type Collection(t) {\n  Collection(\n    name: String,\n    to_json: fn(t) -&gt; Json,\n    decoder: de.Decoder(t),\n    config: Config,\n  )\n}\n A collection, similar to a table in a relational database.\n\n Construct this type to be able to read and write values of type to the\n database.\n\nCollection(\n  name: String,\n  to_json: fn(t) -&gt; Json,\n  decoder: de.Decoder(t),\n  config: Config,\n)\n\nname\n&lt;p&gt;The name of the collection. This needs to be suitable for use in file\npaths. Typically lowercase plural names are recommended, such as “cats”\nand “people”.&lt;/p&gt;\n\nto_json\n&lt;p&gt;A function that encodes a value into JSON, ready to be written to the\ndisc.&lt;/p&gt;\n\ndecoder\n&lt;p&gt;A decoder that transforms the JSON from disc back into the expected\ntype.&lt;/p&gt;\n&lt;p&gt;If you change the structure of your JSON you will need to make sure this\ndecoder supports both the new and the old format, otherwise it will fail\nwhen decoding older JSON.&lt;/p&gt;\n\nconfig\n&lt;p&gt;The configuration for this collection. See the &lt;code&gt;Config&lt;/code&gt; type for\ndetails.&lt;/p&gt;\n\nSynonyms:\nstorail.Collection\nstorail Collection","url":"storail.html#Collection"},{"doc":"storail","title":"Config","content":"pub type Config {\n  Config(storage_path: String)\n}\n The configuration for your storage. This can be different per-collection if\n you prefer, but typically you&#39;d use the same for all collections.\n\nConfig(storage_path: String)\n\nstorage_path\n&lt;p&gt;The directory where data will be written to. Within this directory\nStóráil will create two directories: &lt;code&gt;data&lt;/code&gt; and &lt;code&gt;temporary&lt;/code&gt;. You should\nback-up the &lt;code&gt;data&lt;/code&gt; directory.&lt;/p&gt;\n\nSynonyms:\nstorail.Config\nstorail Config","url":"storail.html#Config"},{"doc":"storail","title":"Key","content":"pub type Key(t) {\n  Key(\n    collection: Collection(t),\n    namespace: List(String),\n    id: String,\n  )\n}\n A pointer into a collection, where an instance could be written to or read\n from. Typically this would be constructed with the `key` and\n `namespaced_key` functions.\n\nKey(\n  collection: Collection(t),\n  namespace: List(String),\n  id: String,\n)\n\ncollection\n&lt;p&gt;The collection this key is for.&lt;/p&gt;\n\nnamespace\n&lt;p&gt;A grouping that this key points into. All objects within a namespace can\nbe queried at once.&lt;/p&gt;\n&lt;p&gt;A use for this may be to create “parents” for object. An “orders”\ncollection may conceptually belong to a “customer” entity, so you may\nchoose to give each order a namespace of &lt;code&gt;[&quot;customer&quot;, customer_id]&lt;/code&gt;.&lt;/p&gt;\n&lt;p&gt;Note that the namespace can be anything, you do not need a “customers”\ncollection to use &lt;code&gt;&quot;customers&quot;&lt;/code&gt; in a namespace list.&lt;/p&gt;\n\nid\n&lt;p&gt;The identifier for the object. These are unique per-namespace.&lt;/p&gt;\n\nSynonyms:\nstorail.Key\nstorail Key","url":"storail.html#Key"},{"doc":"storail","title":"StorailError","content":"pub type StorailError {\n  ObjectNotFound(namespace: List(String), id: String)\n  CorruptJson(path: String, detail: json.DecodeError)\n  FileSystemError(path: String, detail: simplifile.FileError)\n}\n\nObjectNotFound(namespace: List(String), id: String)\n No object was found for the given key, so there was nothing to read.\n\nCorruptJson(path: String, detail: json.DecodeError)\n The object could be read, but it could not be decoded in the desired type.\n\nFileSystemError(path: String, detail: simplifile.FileError)\n There was an error working with the filesystem.\n\nSynonyms:\nstorail.StorailError\nstorail StorailError","url":"storail.html#StorailError"},{"doc":"storail","title":"delete","content":"pub fn delete(key: Key(a)) -&gt; Result(Nil, StorailError)\n Delete an object from the file system.\n\n # Examples\n\n \n pub fn run(cats: Collection(Cat)) {\n   storail.key(cats, &quot;nubi&quot;) |&gt; storail.delete\n   // -&gt; Ok(Nil)\n }\n \n\nSynonyms:\nstorail.delete\nstorail delete","url":"storail.html#delete"},{"doc":"storail","title":"key","content":"pub fn key(collection: Collection(a), id: String) -&gt; Key(a)\n\nSynonyms:\nstorail.key\nstorail key","url":"storail.html#key"},{"doc":"storail","title":"list","content":"pub fn list(\n  collection: Collection(a),\n  namespace: List(String),\n) -&gt; Result(List(String), StorailError)\n List all objects in a namespace.\n\n # Examples\n\n \n pub fn run(cats: Collection(Cat)) {\n   storail.list(cats, [&quot;owner&quot;, &quot;hayleigh&quot;])\n   // -&gt; Ok([&quot;Haskell&quot;, &quot;Agda&quot;])\n }\n \n\nSynonyms:\nstorail.list\nstorail list","url":"storail.html#list"},{"doc":"storail","title":"namespaced_key","content":"pub fn namespaced_key(\n  collection: Collection(a),\n  namespace: List(String),\n  id: String,\n) -&gt; Key(a)\n\nSynonyms:\nstorail.namespaced_key\nstorail namespaced_key","url":"storail.html#namespaced_key"},{"doc":"storail","title":"optional_read","content":"pub fn optional_read(\n  key: Key(a),\n) -&gt; Result(Option(a), StorailError)\n Read an object from the file system, returning `None` if there was no\n object with the given key.\n\n # Examples\n\n \n pub fn run(cats: Collection(Cat)) {\n   storail.key(cats, &quot;nubi&quot;) |&gt; storail.optional_read\n   // -&gt; Ok(Some(Cat(name: &quot;Nubi&quot;, age: 5)))\n\n   storail.key(cats, &quot;mills&quot;) |&gt; storail.optional_read\n   // -&gt; Ok(None)\n }\n \n\nSynonyms:\nstorail.optional_read\nstorail optional_read","url":"storail.html#optional_read"},{"doc":"storail","title":"read","content":"pub fn read(key: Key(a)) -&gt; Result(a, StorailError)\n Read an object from the file system.\n\n # Examples\n\n \n pub fn run(cats: Collection(Cat)) {\n   storail.key(cats, &quot;nubi&quot;) |&gt; storail.read\n   // -&gt; Ok(Cat(name: &quot;Nubi&quot;, age: 5))\n }\n \n\nSynonyms:\nstorail.read\nstorail read","url":"storail.html#read"},{"doc":"storail","title":"read_namespace","content":"pub fn read_namespace(\n  collection: Collection(a),\n  namespace: List(String),\n) -&gt; Result(Dict(String, a), StorailError)\n Read all objects from a namespace.\n\n # Examples\n\n \n pub fn run(cats: Collection(Cat)) {\n   storail.read_namespace(cats, [&quot;owner&quot;, &quot;hayleigh&quot;])\n   // -&gt; Ok([\n   //   Cat(name: &quot;Haskell&quot;, age: 3),\n   //   Cat(name: &quot;Agda&quot;, age: 2),\n   // ])\n }\n \n\nSynonyms:\nstorail.read_namespace\nstorail read_namespace","url":"storail.html#read_namespace"},{"doc":"storail","title":"write","content":"pub fn write(key: Key(a), data: a) -&gt; Result(Nil, StorailError)\n Write an object to the file system.\n\n Writing is done by writing the JSON to the temporary directory and then by\n moving to the data directory. Moving on the same file system is an atomic\n operation for most file systems, so this should avoid data corruption from\n half-written files when writing was interupted by the VM being killed, the\n computer being unplugged, etc.\n\n # Examples\n\n \n pub fn run(cats: Collection(Cat)) {\n   let cat = Cat(name: &quot;Nubi&quot;, age: 5)\n   storail.key(cats, &quot;nubi&quot;) |&gt; storail.write(cat)\n   // -&gt; Ok(Nil)\n }\n \n\nSynonyms:\nstorail.write\nstorail write","url":"storail.html#write"},{"doc":"storail","title":"storail","content":" A super basic on-disc data store that uses JSON files to persist data.\n\n It doesn&#39;t have transactions, MVCC, or anything like that. It&#39;s just\n writing files to disc.\n\n Useful for tiny little projects, and for fun.","url":"storail.html"}]);